(import
  z3
  test
  (chicken file posix)
  (chicken io)
  (chicken port)
  (chicken process)
  (chicken string))

(system "cp ../BSD-LICENCE .")
(system "cp ../chip.jpg .")
(define BSD-LICENCE (with-input-from-file "BSD-LICENCE" read-string))
(system* "gzip -c BSD-LICENCE >BSD-LICENCE.gz")


(test-assert "compressing empty buffer" (z3:encode-buffer ""))
(test-assert "compressing single char buffer" (z3:encode-buffer "0"))

(define fd)
(define z3)

(test-assert "opening file" (set! fd (file-open "BSD-LICENCE.gz" (+ open/trunc open/wronly))))
(test-assert "init" (set! z3 (z3:encode-file fd level: 8)))

(test-assert "writing" (z3:write-encoded z3 (substring BSD-LICENCE 0 100)))
(test-assert "writing (2)" (z3:write-encoded z3 (substring BSD-LICENCE 100)))
(test-assert "writing (3)" (z3:write-encoded z3 #f))
(test-assert "closing" (file-close fd))
(test "testing result" 0 (system "gunzip -c BSD-LICENCE.gz >tmp && cmp tmp BSD-LICENCE"))

(test-assert "opening file" (set! fd (file-open "BSD-LICENCE.gz" open/read)))
(test-assert "init" (set! z3 (z3:decode-file fd)))

(test "reading" BSD-LICENCE (let loop ()
                              (let ((x (z3:read-decoded z3)))
                                (if (eof-object? x)
                                    ""
                                    (string-append x (loop)) ) ) ))
(test-assert "closing file" (file-close fd))

(define dest)
(define r)
(define ze)

(test-assert "compression destination" (set! dest (open-output-string)))
(test-assert "definining compression receiver" (set! r (lambda (x) (display x dest))))
(test-assert "prepare compression" (set! ze (z3:encode-init)))
(test-assert "compressing" (let loop ((c BSD-LICENCE))
                                 (let ((t (z3:encode ze r c)))
                                   (when t
                                     (print* ".")
                                     (loop (substring c t)) ) ) ) )

(define compressed)
(test-assert "compressed" (set! compressed (get-output-string dest)))
(test-assert "compressed size" (conc (string-length BSD-LICENCE) " -> " (string-length compressed)))

(define z)
(test-assert "prepare decompression" (set! z (z3:decode-init)))
(test-assert "decomp. destination" (set! dest (open-output-string)))
(test-assert "decompressing" (let loop ((c compressed))
                                  (let ((t (z3:decode z r c)))
                                    (when t
                                      (print* "(" (string-length c) ")")
                                      (loop (substring c t)) ) ) ) )
(test "comparing results" BSD-LICENCE (get-output-string dest))

(define chip)
(define chipc)
(define chip2)

(test-assert "load jpeg" (set! chip (with-input-from-file "chip.jpg" read-string)))
(test-assert "compressed jpeg" (set! chipc (z3:encode-buffer chip)))
(test-assert "uncompressed jpeg" (set! chip2 (z3:decode-buffer chipc)))
(test "compare uncompressed jpeg to original" chip chip2)

(test-exit)
